@@ -45,6 +45,7 @@ static void albumview_browser_save_myself(GmpcPluginBase *plug);
 static void position_changed(GtkRange *range, gpointer data);
 static void filter_list(GtkEntry *entry, gpointer data);
 void update_view(AlbumViewPlugin *self);
+static void load_list(AlbumViewPlugin *self);
 /**
  * Get/Set enable 
  */
@@ -103,7 +104,7 @@ static void albumview_add(GmpcPluginBrowserIface *plug, GtkWidget *category_tree
 	gtk_list_store_set(GTK_LIST_STORE(model), &iter, 
 			PL3_CAT_TYPE, GMPC_PLUGIN_BASE(plug)->id,
 			PL3_CAT_TITLE,"Album View", 
-			PL3_CAT_ICON_ID, "stylized-no-cover",
+			PL3_CAT_ICON_ID, "albumview",
 			-1);
 	/** 
 	 * remove odl reference if exists 
@@ -259,6 +260,14 @@ static gboolean albumview_expose_event(GtkWidget *widget, GdkEventExpose *event,
     }
     return FALSE;
 }
+#if GTK_CHECK_VERSION(2,16,0)
+static void mod_fill_clear_search_entry(GtkEntry *entry, GtkEntryIconPosition icon_pos, GdkEvent *event, gpointer user_data)
+{
+    if(icon_pos == GTK_ENTRY_ICON_SECONDARY){
+        gtk_entry_set_text(GTK_ENTRY(entry), "");
+    }
+}
+#endif
 static void albumview_init(AlbumViewPlugin *self)
 {
     /** Get an allready exposed widgets to grab theme colors from. */
@@ -278,11 +287,14 @@ static void albumview_init(AlbumViewPlugin *self)
     g_signal_connect(G_OBJECT(self->priv->slider_scale), "value-changed", G_CALLBACK(position_changed), self);
 
     self->priv->filter_entry = gtk_entry_new();
-//    gtk_box_pack_start(GTK_BOX(self->priv->albumview_main_box), self->priv->filter_entry, FALSE, FALSE, 0);
+
+#if GTK_CHECK_VERSION(2,16,0)
+    gtk_entry_set_icon_from_stock(GTK_ENTRY(self->priv->filter_entry), GTK_ENTRY_ICON_SECONDARY, GTK_STOCK_CLEAR);
+    g_signal_connect(GTK_ENTRY(self->priv->filter_entry), "icon-press", G_CALLBACK(mod_fill_clear_search_entry), NULL);
+#endif
     g_signal_connect(G_OBJECT(self->priv->filter_entry),"changed", G_CALLBACK(filter_list), self);
 
     GtkWidget *spin= gtk_spin_button_new_with_range(1, 10, 1);
-//    gtk_scale_set_draw_value(GTK_SCALE(spin), FALSE);
     gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin), self->priv->album_size);
 
     hbox = gtk_hbox_new(FALSE, 6);
@@ -294,12 +306,6 @@ static void albumview_init(AlbumViewPlugin *self)
     gtk_box_pack_start(GTK_BOX(hbox),spin, FALSE, FALSE, 0); 
     gtk_box_pack_end(GTK_BOX(self->priv->albumview_main_box), hbox, FALSE, FALSE, 0);
 
-#if GTK_CHECK_VERSION(2,16,0)
-/*    for(i=4; i<15;i++){
-        gtk_scale_add_mark(GTK_SCALE(spin), (gdouble)i,GTK_POS_TOP, NULL);
-    }*/
-#endif    
-
     gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin), cfg_get_single_value_as_int_with_default(config, "albumview", "zoom-level", 10));
     g_signal_connect(G_OBJECT(spin), "value-changed", G_CALLBACK(album_size_changed), self);
 
@@ -330,6 +336,8 @@ static void albumview_init(AlbumViewPlugin *self)
     g_signal_connect_object(G_OBJECT(self->priv->event_bg), "button-press-event", G_CALLBACK(albumview_button_press_event), self,0);
 
     gtk_widget_show_all(self->priv->albumview_main_box);	
+
+
     /* maintain my own reference to the widget, so it won't get destroyed removing 
      * from view
      */
@@ -346,6 +354,7 @@ static void albumview_selected(GmpcPluginBrowserIface *plug, GtkWidget *containe
     }
     gtk_container_add(GTK_CONTAINER(container), self->priv->albumview_main_box);
     gtk_widget_show(self->priv->albumview_main_box);
+    gtk_widget_show(container);
     gtk_widget_grab_focus(self->priv->event_bg);
 }
 
@@ -355,22 +364,47 @@ static void albumview_unselected(GmpcPluginBrowserIface *plug,GtkWidget *contain
     gtk_container_remove(GTK_CONTAINER(container), self->priv->albumview_main_box);
 }
 
+static void status_changed(GmpcConnection *gmpcconnn,MpdObj *mi, ChangedStatusType type, AlbumViewPlugin *self)
+{
+    if((type&MPD_CST_DATABASE) > 0)
+    {
 
+        if(self->priv->albumview_main_box)
+            load_list(self);
+    }
+}
 
 void albumview_plugin_init(AlbumViewPlugin *self)
 {
-/*  TODO: fix this */
-/*
-    gchar *path = gmpc_plugin_get_data_path(&plugin);
-    gchar *url = g_build_path(G_DIR_SEPARATOR_S,path, "albumview", NULL);
-    debug_printf(DEBUG_WARNING,"Found url: %s\n", url);
+    gchar *path;
+    const gchar * const *paths = g_get_system_data_dirs();
+    int i;
 
+    /* First try the compile time path */
+    path = g_build_filename(PIXMAP_DATA_DIR, NULL);
+    if(path){
+        if(!g_file_test(path, G_FILE_TEST_EXISTS|G_FILE_TEST_IS_DIR))
+        {
+            g_free(path);
+            path = NULL;
+        }
+    }
 
-    gtk_icon_theme_append_search_path(gtk_icon_theme_get_default (),url);
+    for(i=0; path == NULL && paths && paths[i]; i++) {
+        path = g_build_filename(paths[i], "gmpc-albumview", "icons", NULL);
+        if(!g_file_test(path, G_FILE_TEST_EXISTS|G_FILE_TEST_IS_DIR))
+        {
+            g_free(path);
+            path = NULL;
+        }
+    }
+    if(path) {
+        gtk_icon_theme_append_search_path(gtk_icon_theme_get_default (),path);
+    }
+    g_signal_connect_object(G_OBJECT(gmpcconn), "status-changed", G_CALLBACK(status_changed), self, 0);
 
-    g_free(path);	
-    g_free(url);
-    */
+    g_free(path);
+    path = NULL;
 }
 #define TIMER_SUB(start,stop,diff)  diff.tv_usec = stop.tv_usec - start.tv_usec;\
         diff.tv_sec = stop.tv_sec - start.tv_sec;\
@@ -416,22 +450,28 @@ static gint __add_sort(gpointer aa, gpointer bb, gpointer data)
     return -1;
 }
 
-static gboolean load_list_itterate(AlbumViewPlugin *self)
+static gboolean update_progressbar(AlbumViewPlugin *self)
 {
+    gchar *temp = g_strdup_printf("%i of %i albums loaded", self->priv->current_entry, self->priv->max_entries);
+    gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(self->priv->progress_bar), self->priv->current_entry/(double)self->priv->max_entries);
+    gtk_progress_bar_set_text(GTK_PROGRESS_BAR(self->priv->progress_bar), temp);
+    g_free(temp);
+    return FALSE;
+}
+
+static void load_list_itterate(MpdObj *mi, AlbumViewPlugin *self)
+{
+    do{
     MpdData *data2 = NULL;
     self->priv->current_entry++;
     if(self->priv->max_entries>0 && self->priv->current_entry%25 == 0){
-        gchar *temp = g_strdup_printf("%i of %i albums loaded", self->priv->current_entry, self->priv->max_entries);
-        gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(self->priv->progress_bar), self->priv->current_entry/(double)self->priv->max_entries);
-        gtk_progress_bar_set_text(GTK_PROGRESS_BAR(self->priv->progress_bar), temp);
-        g_free(temp);
-        
+        g_idle_add(update_progressbar, self);
     }
     if(self->priv->data)
     {
-        mpd_database_search_field_start(connection, MPD_TAG_ITEM_ARTIST);
-        mpd_database_search_add_constraint(connection, MPD_TAG_ITEM_ALBUM, (self->priv->data)->tag);
-        data2 = mpd_database_search_commit(connection);
+        mpd_database_search_field_start(mi, MPD_TAG_ITEM_ARTIST);
+        mpd_database_search_add_constraint(mi, MPD_TAG_ITEM_ALBUM, (self->priv->data)->tag);
+        data2 = mpd_database_search_commit(mi);
         if(data2)
         {
             mpd_Song *song = mpd_newSong();
@@ -440,11 +480,11 @@ static gboolean load_list_itterate(AlbumViewPlugin *self)
             if(!mpd_data_is_last(data2))
             {
                 /* test if server supports album artist */
-                if(mpd_server_tag_supported(connection, MPD_TAG_ITEM_ALBUM_ARTIST))
+                if(mpd_server_tag_supported(mi, MPD_TAG_ITEM_ALBUM_ARTIST))
                 {
-                    mpd_database_search_field_start(connection, MPD_TAG_ITEM_ALBUM_ARTIST);
-                    mpd_database_search_add_constraint(connection, MPD_TAG_ITEM_ALBUM, (self->priv->data)->tag);
-                    MpdData *data3 = mpd_database_search_commit(connection);
+                    mpd_database_search_field_start(mi, MPD_TAG_ITEM_ALBUM_ARTIST);
+                    mpd_database_search_add_constraint(mi, MPD_TAG_ITEM_ALBUM, (self->priv->data)->tag);
+                    MpdData *data3 = mpd_database_search_commit(mi);
                     if(mpd_data_is_last(data3)){
                         if(strlen(data3->tag) > 0)
                         {
@@ -475,10 +515,16 @@ static gboolean load_list_itterate(AlbumViewPlugin *self)
 
         (self->priv->data) = mpd_data_get_next((self->priv->data));
     }
+    }
+    while(self->priv->data != NULL);
+    self->priv->complete_list = (MpdData *)misc_sort_mpddata(mpd_data_get_first(self->priv->complete_list), (GCompareDataFunc)__add_sort, NULL);
+}
+
+void update_finished(MpdData *data, AlbumViewPlugin *self)
+{
     if(self->priv->data == NULL){
         int items = 0;
         MpdData_real *iter;
-        self->priv->complete_list = (MpdData *)misc_sort_mpddata(mpd_data_get_first(self->priv->complete_list), (GCompareDataFunc)__add_sort, NULL);
         g_log(AV_LOG_DOMAIN, G_LOG_LEVEL_DEBUG,"update view\n");
         gtk_widget_destroy(self->priv->progress_bar);
         self->priv->progress_bar = NULL;
@@ -488,11 +534,11 @@ static gboolean load_list_itterate(AlbumViewPlugin *self)
         filter_list(GTK_ENTRY(self->priv->filter_entry), self);
 
         gtk_widget_grab_focus(self->priv->event_bg);
-        return FALSE;
+     //   return FALSE;
     }
 
-    g_idle_add((GSourceFunc)load_list_itterate, self);
-    return FALSE;
+  //  g_idle_add((GSourceFunc)load_list_itterate, self);
+   // return FALSE;
 }
 
 static void load_list(AlbumViewPlugin *self)
@@ -513,7 +559,7 @@ static void load_list(AlbumViewPlugin *self)
     gtk_widget_set_sensitive(self->priv->filter_entry, FALSE);
     for(iter = data; iter; iter = mpd_data_get_next_real(iter, FALSE)) self->priv->max_entries++;
     self->priv->data= data;
-    g_idle_add((GSourceFunc)load_list_itterate, self);
+    mpd_async_request(update_finished, self, load_list_itterate, self);
 }
 void albumview_connection_changed(GmpcConnection *conn, MpdObj *mi, int connect,void *usedata)
 {
@@ -562,36 +608,32 @@ static void album_replace(GtkWidget *button, mpd_Song *song)
     album_add(button, song);
     mpd_player_play(connection);
 }
-static gboolean album_button_press(GtkWidget *item, GdkEventButton *event, mpd_Song *song)
+static gboolean album_button_press(GtkWidget *image, GtkMenu *menu, mpd_Song *song)
 {   
-    g_log(AV_LOG_DOMAIN, G_LOG_LEVEL_DEBUG,  "button press event %i\n", event->button);
-    if(event->button == 1 && event->type == GDK_BUTTON_PRESS)
-    {
-        GtkWidget *menu = gtk_menu_new();
-        GtkWidget *item;
+    GtkWidget *item;
 
-        item = gtk_image_menu_item_new_with_label("Album information");
-        gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item),
-                gtk_image_new_from_stock(GTK_STOCK_INFO, GTK_ICON_SIZE_MENU));
-        gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-        g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(album_view), song);
+    item = gtk_image_menu_item_new_with_label("Album information");
+    gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item),
+            gtk_image_new_from_stock(GTK_STOCK_INFO, GTK_ICON_SIZE_MENU));
+    gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+    g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(album_view), song);
 
-        item = gtk_image_menu_item_new_from_stock(GTK_STOCK_ADD,NULL);
-        gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-        g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(album_add), song);
+    item = gtk_image_menu_item_new_from_stock(GTK_STOCK_ADD,NULL);
+    gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+    g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(album_add), song);
+
+    /* replace the replace widget */
+    item = gtk_image_menu_item_new_with_label("Replace");
+    gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item),
+            gtk_image_new_from_stock(GTK_STOCK_REDO, GTK_ICON_SIZE_MENU));
+    gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+    g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(album_replace), song);
+
+    item = gtk_separator_menu_item_new();
+    gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+
+    gtk_widget_show(item);
 
-        /* replace the replace widget */
-        item = gtk_image_menu_item_new_with_label("Replace");
-        gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item),
-                gtk_image_new_from_stock(GTK_STOCK_REDO, GTK_ICON_SIZE_MENU));
-        gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-        g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(album_replace), song);
-        gtk_widget_show_all(menu);
-        gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, 0, event->time);
-    }
-    else if (event->button == 2){
-        album_add(NULL, song);
-    }
     return TRUE;
 }
 static GtkWidget * create_button(AlbumViewPlugin *self, MpdData_real *complete_list_iter)
@@ -633,11 +675,15 @@ static GtkWidget * create_button(AlbumViewPlugin *self, MpdData_real *complete_l
 
 
     /* Attach it to the song */
-    complete_list_iter->userdata = g_object_ref_sink(vbox);
+    g_object_add_weak_pointer(vbox,&(complete_list_iter->userdata));
+    complete_list_iter->userdata = vbox;//g_object_ref_sink(vbox);
     complete_list_iter->freefunc = (void *)gtk_widget_destroy;
     g_object_set_data(G_OBJECT(vbox), "item", item);
+    g_signal_connect(G_OBJECT(item), "menu_populate_client", G_CALLBACK(album_button_press), complete_list_iter->song);
+    /*
     g_signal_connect(item, "button-press-event", 
             G_CALLBACK(album_button_press), complete_list_iter->song);
+            */
     return vbox;
 }
 
