@@ -20,7 +20,7 @@
 #include <string.h>
 #include <glade/glade.h>
 #include <gmpc/plugin.h>
-#include <gmpc/gmpc-plugin.h>
+#include <gmpc/gmpc-extras.h>
 #include <config.h>
 /* External pointer + function, there internal from gmpc */
 extern GladeXML *pl3_xml;
@@ -108,7 +108,7 @@ static void extra_playlist_add() {
 	 */
     extraplaylist = gtk_event_box_new();
     /* Set border to fits gmpc's standard  */
-    gtk_container_set_border_width(GTK_CONTAINER(extraplaylist), 3);
+    gtk_container_set_border_width(GTK_CONTAINER(extraplaylist), 0);
 
 	temp = glade_xml_get_widget(pl3_xml, "vbox7");
 	g_object_ref(temp);
