@@ -29,10 +29,10 @@
 #include <metadata.h>
 #include <gmpc-mpddata-model.h>
 #include <gmpc-mpddata-treeview.h>
-#include <gmpc-mpddata-treeview-tooltip.h>
-#include <gmpc-paned-size-group.h>
+#include <gmpc-extras.h>
 #include <sqlite3.h>
 #include "jamendo.h"
+#include <libmpd/debug_printf.h>
 
 extern GladeXML *pl3_xml;
 
@@ -357,6 +357,7 @@ static void jamendo_init()
 	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw), GTK_SHADOW_ETCHED_IN);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
     treeviews[0] = gtk_tree_view_new_with_model(model);
+    gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(treeviews[0]), TRUE);
     gtk_tree_view_set_search_column(GTK_TREE_VIEW(treeviews[0]), MPDDATA_MODEL_COL_SONG_TITLE);
     g_signal_connect(G_OBJECT(treeviews[0]), "button-press-event", G_CALLBACK(jamendo_button_handle_release_event_tag), GINT_TO_POINTER(0));
     /* Add column */
@@ -384,6 +385,7 @@ static void jamendo_init()
 	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw), GTK_SHADOW_ETCHED_IN);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
     treeviews[1] =  gtk_tree_view_new_with_model(model);
+    gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(treeviews[1]), TRUE);
     gmpc_mpd_data_treeview_tooltip_new(GTK_TREE_VIEW(treeviews[1]), META_ARTIST_ART);
     gtk_tree_view_set_search_column(GTK_TREE_VIEW(treeviews[1]), MPDDATA_MODEL_COL_SONG_TITLE);
 	g_signal_connect(G_OBJECT(treeviews[1]), "button-press-event", G_CALLBACK(jamendo_button_handle_release_event_tag), GINT_TO_POINTER(1));
@@ -416,6 +418,7 @@ static void jamendo_init()
 	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw), GTK_SHADOW_ETCHED_IN);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
     treeviews[2] =  gtk_tree_view_new_with_model(model);
+    gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(treeviews[2]), TRUE);
     gmpc_mpd_data_treeview_tooltip_new(GTK_TREE_VIEW(treeviews[2]), META_ALBUM_ART);
     gtk_tree_view_set_search_column(GTK_TREE_VIEW(treeviews[2]), MPDDATA_MODEL_COL_SONG_TITLE);
 
