@@ -29,11 +29,11 @@
 #include <metadata.h>
 #include <gmpc-mpddata-model.h>
 #include <gmpc-mpddata-treeview.h>
-#include <gmpc-mpddata-treeview-tooltip.h>
-#include <gmpc-paned-size-group.h>
+#include <gmpc-extras.h>
 #include <config.h>
 #include <playlist3-messages.h>
 #include "magnatune.h"
+#include <libmpd/debug_printf.h>
 
 extern GladeXML *pl3_xml;
 
@@ -186,6 +186,13 @@ static void magnatune_logo_init()
 	 * open the db 
 	 */
 	magnatune_db_open();
+    {
+        gchar *un = cfg_get_single_value_as_string(config,"magnatune", "username"); 
+        gchar *up = cfg_get_single_value_as_string(config,"magnatune", "password"); 
+        magnatune_set_user_password(un, up);
+        g_free(un);
+        g_free(up);
+    }
 }
 
 
@@ -210,14 +217,19 @@ static void   magnatune_mpd_status_changed(MpdObj *mi, ChangedStatusType what, v
 void magnatune_get_genre_list()
 {
     MpdData *data = NULL;
+    GTimer *timer;
 	data = magnatune_db_get_genre_list();
+
+    timer = g_timer_new();
     gmpc_mpddata_model_set_mpd_data(GMPC_MPDDATA_MODEL(gtk_tree_view_get_model(GTK_TREE_VIEW(treeviews[0]))),data);
+    g_debug("%f seconds elapsed filling genre tree", g_timer_elapsed(timer,NULL)); g_timer_destroy(timer);
 }
 
 static void magnatune_show_album_list(GtkTreeSelection *selection, gpointer user_data)
 {
     MpdData *data = NULL;
     GtkTreeIter piter;
+    GTimer *timer;
     GtkTreeModel *model  = gtk_tree_view_get_model(GTK_TREE_VIEW(treeviews[0]));
     if(gtk_tree_selection_get_selected(selection, &model, &piter))
     {
@@ -227,7 +239,9 @@ static void magnatune_show_album_list(GtkTreeSelection *selection, gpointer user
         
         g_free(genre);
     }
+    timer = g_timer_new();
     gmpc_mpddata_model_set_mpd_data(GMPC_MPDDATA_MODEL(gtk_tree_view_get_model(GTK_TREE_VIEW(treeviews[1]))),data);
+    g_debug("%f seconds elapsed filling artist tree", g_timer_elapsed(timer,NULL)); g_timer_destroy(timer);
 }
 
 static void magnatune_show_song_list(GtkTreeSelection *selection, gpointer user_data)
@@ -236,6 +250,7 @@ static void magnatune_show_song_list(GtkTreeSelection *selection, gpointer user_
     GtkTreeIter piter;
     GtkTreeSelection *select = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeviews[0]));
     GtkTreeModel *model  = gtk_tree_view_get_model(GTK_TREE_VIEW(treeviews[0]));
+    GTimer *timer;
     gchar *genre = NULL, *artist = NULL, *album = NULL;
     if(gtk_tree_selection_get_selected(select, &model, &piter))
     {
@@ -256,12 +271,15 @@ static void magnatune_show_song_list(GtkTreeSelection *selection, gpointer user_
         }
     }
     data = magnatune_db_get_song_list(genre, artist, album,TRUE);
+    timer = g_timer_new();
     gmpc_mpddata_model_set_mpd_data(GMPC_MPDDATA_MODEL(mt_store), data);
+    g_debug("%f seconds elapsed filling song tree", g_timer_elapsed(timer,NULL)); g_timer_destroy(timer);
 }
 
 static void magnatune_show_artist_list(GtkTreeSelection *selection, gpointer user_data)
 {
     GtkTreeIter piter;
+    GTimer *timer;
     MpdData *data = NULL;
     GtkTreeSelection *select = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeviews[0]));
     GtkTreeModel *model  = gtk_tree_view_get_model(GTK_TREE_VIEW(treeviews[0]));
@@ -293,7 +311,9 @@ static void magnatune_show_artist_list(GtkTreeSelection *selection, gpointer use
         }
         g_free(genre);
     }
+    timer = g_timer_new();
     gmpc_mpddata_model_set_mpd_data(GMPC_MPDDATA_MODEL(gtk_tree_view_get_model(GTK_TREE_VIEW(treeviews[2]))),data);
+    g_debug("%f seconds elapsed filling album tree", g_timer_elapsed(timer,NULL)); g_timer_destroy(timer);
 }
 
 static void magnatune_add_album_row_activated(GtkTreeView *tree, GtkTreePath *path, GtkTreeViewColumn *col, gpointer data)
@@ -360,6 +380,7 @@ static void magnatune_init()
 	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw), GTK_SHADOW_ETCHED_IN);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
     treeviews[0] = gtk_tree_view_new_with_model(model);
+    gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(treeviews[0]), TRUE);
 
     gtk_tree_view_set_search_column(GTK_TREE_VIEW(treeviews[0]), MPDDATA_MODEL_COL_SONG_TITLE);
     g_signal_connect(G_OBJECT(treeviews[0]), "button-press-event", G_CALLBACK(magnatune_button_handle_release_event_tag), GINT_TO_POINTER(0));
@@ -388,6 +409,7 @@ static void magnatune_init()
 	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw), GTK_SHADOW_ETCHED_IN);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
     treeviews[1] =  gtk_tree_view_new_with_model(model);
+    gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(treeviews[1]), TRUE);
     gmpc_mpd_data_treeview_tooltip_new(GTK_TREE_VIEW(treeviews[1]), META_ARTIST_ART);
     gtk_tree_view_set_search_column(GTK_TREE_VIEW(treeviews[1]), MPDDATA_MODEL_COL_SONG_TITLE);
 	g_signal_connect(G_OBJECT(treeviews[1]), "button-press-event", G_CALLBACK(magnatune_button_handle_release_event_tag), GINT_TO_POINTER(1));
@@ -420,6 +442,7 @@ static void magnatune_init()
 	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw), GTK_SHADOW_ETCHED_IN);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
     treeviews[2] =  gtk_tree_view_new_with_model(model);
+    gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(treeviews[2]), TRUE);
     gmpc_mpd_data_treeview_tooltip_new(GTK_TREE_VIEW(treeviews[2]), META_ALBUM_ART);
     gtk_tree_view_set_search_column(GTK_TREE_VIEW(treeviews[2]), MPDDATA_MODEL_COL_SONG_TITLE);
 
@@ -701,86 +724,10 @@ static void magnatune_redownload_reload_db()
 			gtk_list_store_remove(GTK_LIST_STORE(model), &citer);
 		}
 		magnatune_download();
-		//magnatune_get_genre_list();
 	}
 	if(path)
 		gtk_tree_path_free(path);
 }
-/*
-static void magnatune_cat_menu_add()
-{
-	GtkTreeIter citer;
-	GtkTreeSelection *sel = gtk_tree_view_get_selection(playlist3_get_category_tree_view());
-	GtkTreeModel *model = (GtkTreeModel *)playlist3_get_category_tree_store();
-	GtkTreePath *tpath;
-	int depth = 0;
-	if(!gtk_tree_selection_get_selected(sel,&model,&citer))
-	{
-		return ;
-	}
-*/	/**
-	 * Get the level we are on.
-	 */
-/*	tpath = gtk_tree_model_get_path(model, &citer);
-	depth = gtk_tree_path_get_depth(tpath);
-	gtk_tree_path_free(tpath);
-*/	/** Genre level */
-/*	if(depth == 2)
-	{
-		GList *list, *node;
-		gchar *genre;
-		gtk_tree_model_get(model, &citer, PL3_CAT_INT_ID,&genre, -1);
-		if(genre)
-		{
-			list = magnatune_db_get_url_list(genre, NULL ,NULL);
-			for(node = list; node; node = node->next)
-			{
-				mpd_playlist_queue_add(connection, node->data);
-			} 
-			mpd_playlist_queue_commit(connection);
-			g_list_foreach(list, (GFunc)g_free, NULL);		
-			g_list_free(list);
-			g_free(genre);
-		}
-	}	
-*/	/** Artist */
-/*	else if(depth == 3)
-	{
-		gchar *artist;
-		gchar *genre;
-		GtkTreeIter iter;
-		if(gtk_tree_model_iter_parent(model, &iter, &citer))
-		{
-			GList *list, *node;
-			gtk_tree_model_get(model, &iter, PL3_CAT_INT_ID,&genre, -1);
-			gtk_tree_model_get(model, &citer, PL3_CAT_INT_ID,&artist, -1);
-			list = magnatune_db_get_url_list(genre, artist,NULL);
-			for(node = list; node; node = node->next)
-			{
-				mpd_playlist_queue_add(connection, node->data);
-			} 
-			mpd_playlist_queue_commit(connection);
-			g_list_foreach(list, (GFunc)g_free, NULL);		
-			g_list_free(list);
-			g_free(genre);
-			g_free(artist);
-		}
-	}
-
-
-
-
-}
-*/
-/*
-static void magnatune_cat_menu_replace()
-{
-	mpd_playlist_clear(connection);
-	magnatune_cat_menu_add();
-	mpd_player_play(connection);
-
-}
-*/
 static int magnatune_cat_menu_popup(GtkWidget *menu, int type, GtkWidget *tree, GdkEventButton *event)
 {
 	GtkWidget *item;
@@ -859,26 +806,12 @@ static int magnatune_button_handle_release_event_tag_add(GtkWidget *button, gpoi
             gtk_tree_model_get(model, &piter, MPDDATA_MODEL_COL_SONG_TITLE,&album,-1);
         }
     }
-    /*
-    list = magnatune_db_get_url_list(genre, artist, album);
-    for(;list; list = list->next)
-    {
-        mpd_playlist_queue_add(connection, list->data);
-    }
-    */
 
     data = magnatune_db_get_song_list(genre,artist,album, TRUE);
     for(data = mpd_data_get_first(data);data;data = mpd_data_get_next(data)) {
         mpd_playlist_queue_add(connection, data->song->file);
     }
     mpd_playlist_queue_commit(connection);
-    /*
-    if(list)
-    {
-        g_list_foreach(list, (GFunc)g_free, NULL);
-        g_list_free(list);
-    }
-    */
     if(genre)
         g_free(genre);
     if(artist)
@@ -958,21 +891,6 @@ static int magnatune_button_release_event(GtkWidget *tree, GdkEventButton *event
 	}
 	return FALSE;
 }
-/*
-static void magnatune_cat_key_press(GtkWidget *tree, GdkEventKey *event, int selected_type)
-{
-	if(selected_type != plugin.id) return; 
-	if(event->state&GDK_CONTROL_MASK && event->keyval == GDK_Insert)
-	{
-		magnatune_cat_menu_replace();
-	}
-	else if(event->keyval == GDK_Insert)
-	{
-		magnatune_cat_menu_add();	
-	}
-
-}
-*/
 static int magnatune_key_press(GtkWidget *tree, GdkEventKey *event)
 {
 	if(event->state&GDK_CONTROL_MASK && event->keyval == GDK_Insert)
@@ -1073,6 +991,60 @@ static const gchar *magnatune_get_translation_domain(void)
     return GETTEXT_PACKAGE;
 }
 
+static void magnatune_uentry_changed(GtkEntry *entry)
+{
+    const gchar *text = gtk_entry_get_text(entry);
+    cfg_set_single_value_as_string(config,"magnatune", "username", text); 
+}
+static void magnatune_pentry_changed(GtkEntry *entry)
+{
+    const gchar *text = gtk_entry_get_text(entry);
+    cfg_set_single_value_as_string(config,"magnatune", "password", text); 
+}
+
+
+void magnatune_pref_construct (GtkWidget *container)
+{
+    GtkWidget *table;
+    GtkWidget *label, *uentry, *pentry;
+    gchar *un = cfg_get_single_value_as_string(config,"magnatune", "username"); 
+    gchar *up = cfg_get_single_value_as_string(config,"magnatune", "password"); 
+
+    table = gtk_table_new(3, 2, FALSE);
+
+    /* Password */
+    label = gtk_label_new(_("Username"));
+    gtk_table_attach(GTK_TABLE(table), label, 0,1, 0,1, GTK_EXPAND|GTK_FILL, GTK_SHRINK|GTK_FILL, 0, 0);
+    uentry = gtk_entry_new();
+    if(un) gtk_entry_set_text(GTK_ENTRY(uentry), un);
+    g_signal_connect(G_OBJECT(uentry), "changed", G_CALLBACK(magnatune_uentry_changed), NULL);
+    gtk_table_attach(GTK_TABLE(table), uentry, 1,2, 0,1,GTK_EXPAND|GTK_FILL, GTK_SHRINK|GTK_FILL, 0, 0);
+
+
+    label = gtk_label_new(_("Password"));
+    gtk_table_attach(GTK_TABLE(table), label, 0,1, 1,2,GTK_EXPAND|GTK_FILL, GTK_SHRINK|GTK_FILL, 0, 0);
+
+    pentry = gtk_entry_new();
+    gtk_entry_set_visibility(GTK_ENTRY(pentry), FALSE);
+    if(up) gtk_entry_set_text(GTK_ENTRY(pentry), up);
+    g_signal_connect(G_OBJECT(pentry), "changed", G_CALLBACK(magnatune_pentry_changed), NULL);
+    gtk_table_attach(GTK_TABLE(table), pentry, 1,2, 1,2, GTK_EXPAND|GTK_FILL, GTK_SHRINK|GTK_FILL, 0, 0);
+
+    g_free(un);
+    g_free(up);
+    gtk_container_add(GTK_CONTAINER(container), table);
+    gtk_widget_show_all(container);
+}
+void magnatune_pref_destroy (GtkWidget *container)
+{
+    GtkWidget *child = gtk_bin_get_child(GTK_BIN(container));
+    if(child) gtk_widget_destroy(child);
+}
+
+gmpcPrefPlugin magnatune_gpp = {
+    magnatune_pref_construct,
+    magnatune_pref_destroy
+};
 gmpcPlugin plugin = {
 	.name                       = N_("Magnatune Stream Browser"),
 	.version                    = {PLUGIN_MAJOR_VERSION,PLUGIN_MINOR_VERSION,PLUGIN_MICRO_VERSION},
@@ -1087,6 +1059,7 @@ gmpcPlugin plugin = {
 	.browser                    = &magnatune_gbp,
 	.metadata                   = &magnatune_cover,
 
+    .pref                       = &magnatune_gpp,
     /* get/set enabled  */
 	.get_enabled                = magnatune_get_enabled,
 	.set_enabled                = magnatune_set_enabled,
@@ -1094,5 +1067,3 @@ gmpcPlugin plugin = {
 
     .get_translation_domain = magnatune_get_translation_domain
 };
-
-
