@@ -38,6 +38,9 @@
 #include <libxml/parser.h>
 #include <libxml/tree.h>
 
+#include <libmpd/debug_printf.h>
+
+#include <libsoup/soup.h>
 static void wp_add(GtkWidget *cat_tree);
 static void wp_selected(GtkWidget *container);
 static void wp_unselected(GtkWidget *container);
@@ -291,7 +294,8 @@ static WebKitNavigationResponse wp_navigation_requested(WebKitWebView *web_view,
 		WebKitWebFrame *frame, WebKitNetworkRequest *request,
 		gpointer user_data)
 {
-	gchar *uri, *decoded_uri;
+    const gchar *uri;
+    gchar *decoded_uri;
 	GError *error = NULL;
 	GdkScreen *screen;
 
@@ -368,7 +372,7 @@ void wp_init()
 void wp_add(GtkWidget *cat_tree)
 {
 	GtkTreePath *path = NULL;
-	GtkTreeStore *pl3_tree = (GtkTreeStore *)gtk_tree_view_get_model(GTK_TREE_VIEW(cat_tree));
+	GtkListStore *pl3_tree = (GtkListStore *)gtk_tree_view_get_model(GTK_TREE_VIEW(cat_tree));
 	GtkTreeIter iter;
 	if(!cfg_get_single_value_as_int_with_default(config, "wp-plugin", "enable", 0)) return;
 	gtk_list_store_append(pl3_tree, &iter);
